@@ -12,8 +12,8 @@ url=$(git config bash-completion.docker-hub-trigger-url)
 
 test "$(git symbolic-ref --short HEAD 2>/dev/null)" = master
 
-git diff-tree -r --name-only --no-commit-id HEAD | \
-    grep -qxE 'completions/Makefile\.am|test/requirements\.txt'
+git diff-tree -r --name-only --no-commit-id HEAD |
+    grep -qxE 'test/test-cmd-list\.txt'
 
 curl \
     --silent --show-error \
