@@ -1,7 +1,9 @@
 #!/bin/sh -eu
 
-targetdir="$1"; shift
-target="$1"; shift
+targetdir="$1"
+shift
+target="$1"
+shift
 
 for file in "$@"; do
     rm -f "$targetdir/$file"
