@@ -2,6 +2,6 @@ import pytest
 
 
 class TestBison:
-    @pytest.mark.complete("bison --")
+    @pytest.mark.complete("bison --", require_cmd=True)
     def test_1(self, completion):
         assert completion
