@@ -9,7 +9,7 @@ class TestCd:
 
     @pytest.mark.complete("cd fo", env=dict(CDPATH="shared/default"))
     def test_2(self, completion):
-        assert completion == "foo.d/"
+        assert completion == "o.d/"
 
     @pytest.mark.complete("cd fo")
     def test_3(self, completion):
@@ -20,3 +20,7 @@ class TestCd:
     )
     def test_4(self, completion):
         assert not completion  # No subdirs nor CDPATH
+
+    @pytest.mark.complete("cd shared/default/", trail="foo")
+    def test_dir_at_point(self, completion):
+        assert completion == ["bar bar.d/", "foo.d/"]
