@@ -6,7 +6,7 @@ class TestFirefox:
     def test_1(self, completion):
         assert completion
 
-    @pytest.mark.complete("firefox -")
+    @pytest.mark.complete("firefox -", require_cmd=True)
     def test_2(self, completion):
         assert completion
         assert not completion.endswith(" ")
