@@ -2,6 +2,6 @@ import pytest
 
 
 class TestHwclock:
-    @pytest.mark.complete("hwclock -")
+    @pytest.mark.complete("hwclock -", require_cmd=True)
     def test_1(self, completion):
         assert completion
