@@ -2,7 +2,7 @@ import pytest
 
 
 class TestIperf3:
-    @pytest.mark.complete("iperf3 ")
+    @pytest.mark.complete("iperf3 ", require_cmd=True)
     def test_1(self, completion):
         assert completion
 
@@ -10,11 +10,11 @@ class TestIperf3:
     def test_2(self, completion):
         assert completion
 
-    @pytest.mark.complete("iperf3 --client foo --")
+    @pytest.mark.complete("iperf3 --client foo --", require_cmd=True)
     def test_3(self, completion):
         assert completion
         assert "--daemon" not in completion
 
-    @pytest.mark.complete("iperf3 --server --")
+    @pytest.mark.complete("iperf3 --server --", require_cmd=True)
     def test_4(self, completion):
         assert "--daemon" in completion
