@@ -3,7 +3,7 @@ import pytest
 
 class TestLvchange:
     @pytest.mark.complete(
-        "lvchange --", skipif="! lvchange --help &>/dev/null"
+        "lvchange --", require_cmd=True, xfail="! lvchange --help &>/dev/null"
     )
     def test_1(self, completion):
         assert completion
