@@ -6,12 +6,12 @@ import pytest
 class TestModprobe:
     @pytest.mark.complete("modprobe --al")
     def test_1(self, completion):
-        assert completion == "--all"
+        assert completion == "l"
 
     # "in": intel*, ...
     @pytest.mark.complete(
         "modprobe in",
-        skipif="! ls /lib/modules/%s &>/dev/null"
+        xfail="! ls /lib/modules/%s &>/dev/null"
         % subprocess.check_output(
             "uname -r 2>/dev/null || " "echo non-existent-kernel", shell=True
         )
