@@ -6,7 +6,7 @@ class TestMypy:
     def test_1(self, completion):
         assert completion
 
-    @pytest.mark.complete("mypy --", require_cmd=True)
+    @pytest.mark.complete("mypy --", require_cmd=True, require_longopt=True)
     def test_2(self, completion):
         assert completion
 
