@@ -6,6 +6,6 @@ class TestNsupdate:
     def test_1(self, completion):
         assert completion
 
-    @pytest.mark.complete("nsupdate -")
+    @pytest.mark.complete("nsupdate -", require_cmd=True)
     def test_2(self, completion):
         assert completion
