@@ -6,6 +6,6 @@ class TestPngfix:
     def test_1(self, completion):
         assert completion
 
-    @pytest.mark.complete("pngfix -")
+    @pytest.mark.complete("pngfix -", require_cmd=True)
     def test_2(self, completion):
         assert completion
