@@ -6,7 +6,7 @@ class TestPv:
     def test_1(self, completion):
         assert completion
 
-    @pytest.mark.complete("pv -")
+    @pytest.mark.complete("pv -", require_cmd=True)
     def test_2(self, completion):
         assert completion
 
