@@ -6,6 +6,6 @@ class TestTracepath:
     def test_1(self, completion):
         assert completion
 
-    @pytest.mark.complete("tracepath -")
+    @pytest.mark.complete("tracepath -", require_cmd=True)
     def test_2(self, completion):
         assert completion
