@@ -2,6 +2,6 @@ import pytest
 
 
 class TestUrlsnarf:
-    @pytest.mark.complete("urlsnarf -")
+    @pytest.mark.complete("urlsnarf -", require_cmd=True)
     def test_1(self, completion):
         assert completion
