@@ -6,6 +6,6 @@ class TestWget:
     def test_1(self, completion):
         assert not completion
 
-    @pytest.mark.complete("wget --h")
+    @pytest.mark.complete("wget --s", require_cmd=True)
     def test_2(self, completion):
         assert completion
